==> v1.12.9.zip: wcs/tests/test_register.py <==
import hashlib
import re
import shutil
from quixote import cleanup

from wcs.qommon.http_request import HTTPRequest
from wcs.qommon.ident.password_accounts import PasswordAccount

from utilities import get_app, create_temporary_pub, emails

def setup_module(module):
    cleanup()
    global pub
    pub = create_temporary_pub()

    req = HTTPRequest(None, {'SCRIPT_NAME': '/', 'SERVER_NAME': 'example.net'})
    pub.set_app_dir(req)
    pub.cfg['identification'] = {'methods': ['password']}
    pub.cfg['identities'] = {'creation': 'self'}
    pub.write_cfg()

def teardown_module(module):
    global pub
    shutil.rmtree(pub.APP_DIR)

def test_no_user_registration():
    # makes sure the page is not published unless configured
    app = get_app(pub)
    pub.cfg['identification'] = {'methods': ['password']}
    pub.cfg['identities'] = {'creation': 'admin'}
    pub.write_cfg()
    app.get('/register/', status=404)
    pub.cfg['identities'] = {'creation': 'self'}
    pub.write_cfg()

def test_link_on_login_page():
    app = get_app(pub)
    page = app.get('/login/')
    assert '/register/' in page.body

def test_no_password():
    app = get_app(pub)
    page = app.get('/register/')
    register_form = page.forms[0]
    assert 'username' in register_form.fields
    assert 'password' not in register_form.fields

def test_user_registration_mismatch():
    pub.cfg['passwords'] = {'generate': False}
    pub.write_cfg()
    app = get_app(pub)
    page = app.get('/register/')
    register_form = page.forms[0]
    register_form['username'] = 'foo'
    register_form['password'] = 'bar'
    register_form['password2'] = 'baz'
    resp = register_form.submit()
    assert 'Passwords do not match' in resp.body

def do_user_registration(username='foo', password='bar'):
    initial_user_count = pub.user_class.count()
    initial_account_count = PasswordAccount.count()
    app = get_app(pub)
    page = app.get('/register/')
    register_form = page.forms[0]
    register_form['username'] = username
    if password is not None:
        register_form['password'] = password
        register_form['password2'] = password
    resp = register_form.submit()
    assert resp.status_int == 302
    assert resp.location == 'http://example.net/login'

    assert pub.user_class.count() == initial_user_count + 1
    assert PasswordAccount.count() == initial_account_count + 1

    account = PasswordAccount.get(username)
    user = account.get_user()
    if password is not None:
        user2 = PasswordAccount.get_with_credentials(username, password)
        assert user.id == user2.id

def test_user_registration():
    pub.user_class.wipe()
    PasswordAccount.wipe()
    pub.cfg['passwords'] = {'generate': False}
    pub.write_cfg()
    do_user_registration()

    account = PasswordAccount.get('foo')
    assert account.password == 'bar' # check it's in clear text

def test_user_password_hashing():
    pub.user_class.wipe()
    PasswordAccount.wipe()
    pub.cfg['passwords'] = {'generate': False, 'hashing_algo': 'sha256'}
    pub.write_cfg()
    do_user_registration()

    account = PasswordAccount.get('foo')
    assert account.password == hashlib.sha256('bar').hexdigest()

def test_admin_notification():
    pub.cfg['identities'] = {'creation': 'self', 'notify-on-register': True}
    pub.write_cfg()
    pub.user_class.wipe()
    PasswordAccount.wipe()
    user = pub.user_class(name='admin')
    user.is_admin = True
    user.email = 'admin@localhost'
    user.store()

    pub.cfg['passwords'] = {'generate': False}
    pub.write_cfg()
    do_user_registration()

    assert emails.get('New Registration')
    assert emails.get('New Registration').get('kwargs').get('email_rcpt') == ['admin@localhost']
    assert emails.get('New Registration').get('kwargs').get('fire_and_forget') == True

def test_user_notification():
    pub.cfg['identities'] = {'creation': 'self', 'notify-on-register': False,
            'email-as-username': True}
    pub.write_cfg()
    pub.user_class.wipe()
    PasswordAccount.wipe()
    user = pub.user_class(name='admin')
    user.is_admin = True
    user.email = 'admin@localhost'
    user.store()

    pub.cfg['passwords'] = {'generate': True}
    pub.write_cfg()
    do_user_registration(username='foo@localhost', password=None)

    account = PasswordAccount.get('foo@localhost')

    assert emails.get('Welcome to example.net')
    assert emails.get('Welcome to example.net').get('kwargs').get('email_rcpt') == 'foo@localhost'
    assert account.password in emails.get('Welcome to example.net').get('args')[0]

def test_user_login():
    pub.cfg['identities'] = {'creation': 'self', 'notify-on-register': False}
    pub.user_class.wipe()
    PasswordAccount.wipe()
    pub.cfg['passwords'] = {'generate': False, 'hashing_algo': 'sha256'}
    pub.write_cfg()
    do_user_registration()

    # wrong password
    app = get_app(pub)
    resp = app.get('/login/')
    resp.forms[0]['username'] = 'foo'
    resp.forms[0]['password'] = 'foo'
    resp = resp.forms[0].submit()
    assert 'Invalid credentials' in resp.body

    # correct passwod
    app = get_app(pub)
    resp = app.get('/login/')
    resp.forms[0]['username'] = 'foo'
    resp.forms[0]['password'] = 'bar'
    resp = resp.forms[0].submit()
    assert resp.location == 'http://example.net/'

def test_forgotten():
    pub.cfg['identities'] = {'creation': 'self', 'notify-on-register': False}
    pub.user_class.wipe()
    PasswordAccount.wipe()
    pub.cfg['passwords'] = {'generate': False, 'hashing_algo': 'sha256'}
    pub.write_cfg()
    do_user_registration()

    app = get_app(pub)
    resp = app.get('/login/')
    assert '/ident/password/forgotten' in resp.body

    resp = app.get('/ident/password/forgotten')
    resp.forms[0]['username'] = 'bar' # this account doesn't exist
    resp = resp.forms[0].submit()
    assert 'There is no user with that name or it has no email contact.'

    resp = app.get('/ident/password/forgotten')
    resp.forms[0]['username'] = 'foo' # this account doesn't have an email
    resp = resp.forms[0].submit()
    assert 'There is no user with that name or it has no email contact.'

    user = pub.user_class.get(1)
    user.email = 'foo@localhost'
    user.store()

    resp = app.get('/ident/password/forgotten')
    resp.forms[0]['username'] = 'foo'
    resp = resp.forms[0].submit()
    assert 'A token for changing your password has been emailed to you.' in resp.body

    assert emails.get('Change Password Request')
    assert emails.get('Change Password Request')['kwargs']['email_rcpt'] == 'foo@localhost'
    body = emails.get('Change Password Request')['args'][0]

    confirm_urls = re.findall(r'http://.*\w',  body)
    assert 'a=cfmpw' in confirm_urls[0]
    assert 'a=cxlpw' in confirm_urls[1]

    # cancel request
    resp = app.get(confirm_urls[1])
    assert 'Your request has been cancelled' in resp.body

    resp = app.get(confirm_urls[1])
    assert 'The token you submitted does not exist' in resp.body

    # new forgotten request
    resp = app.get('/ident/password/forgotten')
    resp.forms[0]['username'] = 'foo'
    resp = resp.forms[0].submit()
    assert 'A token for changing your password has been emailed to you.' in resp.body

    body = emails.get('Change Password Request')['args'][0]
    confirm_urls = re.findall(r'http://.*\w',  body)
    assert 'a=cfmpw' in confirm_urls[0]
    assert 'a=cxlpw' in confirm_urls[1]

    resp = app.get(confirm_urls[0])
    assert 'New password sent by email' in resp.body
    assert emails.get('Your new password')

    # check new password is working
    new_password = re.findall('password: (.*)\n', emails.get('Your new password')['args'][0])[0]
    resp = app.get('/login/')
    resp.forms[0]['username'] = 'foo'
    resp.forms[0]['password'] = new_password
    resp = resp.forms[0].submit()
    assert resp.status_int == 302

==> v1.12.12.zip: wcs/tests/test_register.py <==
import hashlib
import re
import shutil
from quixote import cleanup

from wcs.qommon.http_request import HTTPRequest
from wcs.qommon.ident.password_accounts import PasswordAccount

from utilities import get_app, create_temporary_pub, emails

def setup_module(module):
    cleanup()
    global pub
    pub = create_temporary_pub()

    req = HTTPRequest(None, {'SCRIPT_NAME': '/', 'SERVER_NAME': 'example.net'})
    pub.set_app_dir(req)
    pub.cfg['identification'] = {'methods': ['password']}
    pub.cfg['identities'] = {'creation': 'self'}
    pub.write_cfg()

def teardown_module(module):
    global pub
    shutil.rmtree(pub.APP_DIR)

def test_no_user_registration():
    # makes sure the page is not published unless configured
    app = get_app(pub)
    pub.cfg['identification'] = {'methods': ['password']}
    pub.cfg['identities'] = {'creation': 'admin'}
    pub.write_cfg()
    app.get('/register/', status=404)
    pub.cfg['identities'] = {'creation': 'self'}
    pub.write_cfg()

def test_link_on_login_page():
    app = get_app(pub)
    page = app.get('/login/')
    assert '/register/' in page.body

def test_no_password():
    app = get_app(pub)
    page = app.get('/register/')
    register_form = page.forms[0]
    assert 'username' in register_form.fields
    assert 'password' not in register_form.fields

def test_user_registration_mismatch():
    pub.cfg['passwords'] = {'generate': False}
    pub.write_cfg()
    app = get_app(pub)
    page = app.get('/register/')
    register_form = page.forms[0]
    register_form['username'] = 'foo'
    register_form['password$pwd1'] = 'bar'
    register_form['password$pwd2'] = 'baz'
    resp = register_form.submit()
    assert 'Passwords do not match' in resp.body

def do_user_registration(username='foo', password='bar'):
    initial_user_count = pub.user_class.count()
    initial_account_count = PasswordAccount.count()
    app = get_app(pub)
    page = app.get('/register/')
    register_form = page.forms[0]
    register_form['username'] = username
    if password is not None:
        register_form['password$pwd1'] = 'bar'
        register_form['password$pwd2'] = 'bar'
    resp = register_form.submit()
    assert resp.status_int == 302
    assert resp.location == 'http://example.net/login'

    assert pub.user_class.count() == initial_user_count + 1
    assert PasswordAccount.count() == initial_account_count + 1

    account = PasswordAccount.get(username)
    user = account.get_user()
    if password is not None:
        user2 = PasswordAccount.get_with_credentials(username, password)
        assert user.id == user2.id

def test_user_registration():
    pub.user_class.wipe()
    PasswordAccount.wipe()
    pub.cfg['passwords'] = {'generate': False}
    pub.write_cfg()
    do_user_registration()

    account = PasswordAccount.get('foo')
    assert account.password == 'bar' # check it's in clear text

def test_user_password_hashing():
    pub.user_class.wipe()
    PasswordAccount.wipe()
    pub.cfg['passwords'] = {'generate': False, 'hashing_algo': 'sha256'}
    pub.write_cfg()
    do_user_registration()

    account = PasswordAccount.get('foo')
    assert account.password == hashlib.sha256('bar').hexdigest()

def test_admin_notification():
    pub.cfg['identities'] = {'creation': 'self', 'notify-on-register': True}
    pub.write_cfg()
    pub.user_class.wipe()
    PasswordAccount.wipe()
    user = pub.user_class(name='admin')
    user.is_admin = True
    user.email = 'admin@localhost'
    user.store()

    pub.cfg['passwords'] = {'generate': False}
    pub.write_cfg()
    do_user_registration()

    assert emails.get('New Registration')
    assert emails.get('New Registration').get('kwargs').get('email_rcpt') == ['admin@localhost']
    assert emails.get('New Registration').get('kwargs').get('fire_and_forget') == True

def test_user_notification():
    pub.cfg['identities'] = {'creation': 'self', 'notify-on-register': False,
            'email-as-username': True}
    pub.write_cfg()
    pub.user_class.wipe()
    PasswordAccount.wipe()
    user = pub.user_class(name='admin')
    user.is_admin = True
    user.email = 'admin@localhost'
    user.store()

    pub.cfg['passwords'] = {'generate': True}
    pub.write_cfg()
    do_user_registration(username='foo@localhost', password=None)

    account = PasswordAccount.get('foo@localhost')

    assert emails.get('Welcome to example.net')
    assert emails.get('Welcome to example.net').get('kwargs').get('email_rcpt') == 'foo@localhost'
    assert account.password in emails.get('Welcome to example.net').get('args')[0]

def test_user_login():
    pub.cfg['identities'] = {'creation': 'self', 'notify-on-register': False}
    pub.user_class.wipe()
    PasswordAccount.wipe()
    pub.cfg['passwords'] = {'generate': False, 'hashing_algo': 'sha256'}
    pub.write_cfg()
    do_user_registration()

    # wrong password
    app = get_app(pub)
    resp = app.get('/login/')
    resp.forms[0]['username'] = 'foo'
    resp.forms[0]['password'] = 'foo'
    resp = resp.forms[0].submit()
    assert 'Invalid credentials' in resp.body

    # correct passwod
    app = get_app(pub)
    resp = app.get('/login/')
    resp.forms[0]['username'] = 'foo'
    resp.forms[0]['password'] = 'bar'
    resp = resp.forms[0].submit()
    assert resp.location == 'http://example.net/'

def test_forgotten():
    pub.cfg['identities'] = {'creation': 'self', 'notify-on-register': False}
    pub.user_class.wipe()
    PasswordAccount.wipe()
    pub.cfg['passwords'] = {'generate': False, 'hashing_algo': 'sha256'}
    pub.write_cfg()
    do_user_registration()

    app = get_app(pub)
    resp = app.get('/login/')
    assert '/ident/password/forgotten' in resp.body

    resp = app.get('/ident/password/forgotten')
    resp.forms[0]['username'] = 'bar' # this account doesn't exist
    resp = resp.forms[0].submit()
    assert 'There is no user with that name or it has no email contact.'

    resp = app.get('/ident/password/forgotten')
    resp.forms[0]['username'] = 'foo' # this account doesn't have an email
    resp = resp.forms[0].submit()
    assert 'There is no user with that name or it has no email contact.'

    user = pub.user_class.get(1)
    user.email = 'foo@localhost'
    user.store()

    resp = app.get('/ident/password/forgotten')
    resp.forms[0]['username'] = 'foo'
    resp = resp.forms[0].submit()
    assert 'A token for changing your password has been emailed to you.' in resp.body

    assert emails.get('Change Password Request')
    assert emails.get('Change Password Request')['kwargs']['email_rcpt'] == 'foo@localhost'
    body = emails.get('Change Password Request')['args'][0]

    confirm_urls = re.findall(r'http://.*\w',  body)
    assert 'a=cfmpw' in confirm_urls[0]
    assert 'a=cxlpw' in confirm_urls[1]

    # cancel request
    resp = app.get(confirm_urls[1])
    assert 'Your request has been cancelled' in resp.body

    resp = app.get(confirm_urls[1])
    assert 'The token you submitted does not exist' in resp.body

    # new forgotten request
    resp = app.get('/ident/password/forgotten')
    resp.forms[0]['username'] = 'foo'
    resp = resp.forms[0].submit()
    assert 'A token for changing your password has been emailed to you.' in resp.body

    body = emails.get('Change Password Request')['args'][0]
    confirm_urls = re.findall(r'http://.*\w',  body)
    assert 'a=cfmpw' in confirm_urls[0]
    assert 'a=cxlpw' in confirm_urls[1]

    resp = app.get(confirm_urls[0])
    assert 'New password sent by email' in resp.body
    assert emails.get('Your new password')

    # check new password is working
    new_password = re.findall('password: (.*)\n', emails.get('Your new password')['args'][0])[0]
    resp = app.get('/login/')
    resp.forms[0]['username'] = 'foo'
    resp.forms[0]['password'] = new_password
    resp = resp.forms[0].submit()
    assert resp.status_int == 302
